﻿using System;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Attributes {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  [AttributeUsage(AttributeTargets.Class)]
  public class PushExpressionAttribute : Attribute {
    public readonly StackTypes StackType;
    public readonly StackTypes AdditionalStackDependencies;
    public readonly string ExpressionName;
    public readonly bool ManipulatesExec;
    public readonly uint ExecIn;

    /// <summary>
    /// Determines if the expression is visible to the user during the configuration phase for example.
    /// </summary>
    public readonly bool IsHidden;

    public PushExpressionAttribute(
      StackTypes stackType,
      string expressionName,
      StackTypes additionalStackDependencies = default(StackTypes),
      uint execIn = 0,
      bool isHidden = false) {
      StackType = stackType;
      AdditionalStackDependencies = additionalStackDependencies;
      ExpressionName = expressionName;
      ExecIn = execIn;
      IsHidden = isHidden;
      ManipulatesExec = stackType == StackTypes.Exec || AdditionalStackDependencies.HasFlag(StackTypes.Exec);
    }
  }
}
